@@ -1,39 +1,68 @@
+import abc
 import asyncio
 import functools
 import json
 from enum import Enum
-from itertools import chain
+from io import FileIO
 from typing import (
     Any,
-    AsyncGenerator,
+    AsyncIterable,
     AsyncIterator,
     Awaitable,
     Callable,
     Dict,
     Generic,
+    Iterable,
     Iterator,
     Mapping,
-    MutableSequence,
-    Optional,
     Sequence,
     Tuple,
     TypeVar,
     Union,
 )
+from typing import cast as typing_cast
 from urllib.parse import parse_qsl, quote_plus
 
+from .concurrency import run_in_threadpool
 from .datastructures import URL, Address, FormData, Headers, QueryParams
 from .exceptions import HTTPException
 from .formparsers import AsyncMultiPartParser
 from .requests import MoreInfoFromHeaderMixin
-from .responses import BaseFileResponse, BaseResponse
-from .routing import BaseHosts, BaseRouter
-from .typing import ASGIApp, JSONable, Message, Receive, Scope, Send, ServerSentEvent
+from .responses import BaseFileResponse, BaseResponse, build_bytes_from_sse
+from .routing import BaseHosts, BaseRouter, BaseSubpaths
+from .typing import ASGIApp, Message, Receive, Scope, Send, ServerSentEvent
 from .utils import cached_property
 
 
+async def send_http_start(
+    send: Send, status_code: int, headers: Iterable[Tuple[bytes, bytes]] = None
+) -> None:
+    """
+    helper function for send http.response.start
+
+    https://asgi.readthedocs.io/en/latest/specs/www.html#response-start-send-event
+    """
+    message = {"type": "http.response.start", "status": status_code}
+    if headers is not None:
+        message["headers"] = headers
+    await send(message)
+
+
+async def send_http_body(
+    send: Send, body: bytes = b"", *, more_body: bool = False
+) -> None:
+    """
+    helper function for send http.response.body
+
+    https://asgi.readthedocs.io/en/latest/specs/www.html#response-body-send-event
+    """
+    await send({"type": "http.response.body", "body": body, "more_body": more_body})
+
+
 class ClientDisconnect(Exception):
-    pass
+    """
+    HTTP connection disconnected.
+    """
 
 
 async def empty_receive() -> Message:
@@ -44,10 +73,12 @@ async def empty_send(message: Message) -> None:
     raise NotImplementedError("Send channel has not been made available")
 
 
-class HTTPConnection(Mapping, MoreInfoFromHeaderMixin):
+class HTTPConnection(Mapping[str, Any], MoreInfoFromHeaderMixin):
     """
-    A base class for incoming HTTP connections, that is used to provide
-    any functionality that is common to both `Request` and `WebSocket`.
+    A base class for incoming HTTP connections.
+
+    It is a valid Mapping type that allows you to directly
+    access the values in any ASGI `scope` dictionary.
     """
 
     def __init__(
@@ -57,7 +88,7 @@ class HTTPConnection(Mapping, MoreInfoFromHeaderMixin):
         self._send = send
         self._receive = receive
 
-    def __getitem__(self, key: str) -> str:
+    def __getitem__(self, key: str) -> Any:
         return self._scope[key]
 
     def __iter__(self) -> Iterator[str]:
@@ -68,24 +99,47 @@ class HTTPConnection(Mapping, MoreInfoFromHeaderMixin):
 
     @cached_property
     def client(self) -> Address:
+        """
+        Client's IP and Port.
+
+        Note that this depends on the "client" value given by
+        the ASGI Server, and is not necessarily accurate.
+        """
         host, port = self.get("client") or (None, None)
         return Address(host=host, port=port)
 
     @cached_property
     def url(self) -> URL:
+        """
+        The full URL of this request.
+        """
         return URL(scope=self._scope)
 
     @cached_property
     def path_params(self) -> Dict[str, Any]:
+        """
+        The path parameters parsed by the framework.
+        """
         return self.get("path_params", {})
 
     @cached_property
     def query_params(self) -> QueryParams:
+        """
+        Query parameter. It is a multi-value mapping.
+        """
         return QueryParams(self["query_string"])
 
     @cached_property
     def headers(self) -> Headers:
-        return Headers(scope=self._scope)
+        """
+        A read-only case-independent mapping.
+
+        Note that in its internal storage, all keys are in lower case.
+        """
+        return Headers(
+            (key.decode("latin-1"), value.decode("latin-1"))
+            for key, value in self._scope["headers"]
+        )
 
 
 class Request(HTTPConnection):
@@ -99,9 +153,19 @@ class Request(HTTPConnection):
 
     @cached_property
     def method(self) -> str:
+        """
+        HTTP method. Uppercase string.
+        """
         return self._scope["method"]
 
-    async def stream(self) -> AsyncGenerator[bytes, None]:
+    async def stream(self) -> AsyncIterator[bytes]:
+        """
+        Streaming read request body. e.g. `async for chunk in request.stream(): ...`
+
+        If you access `.stream()` then the byte chunks are provided
+        without storing the entire body to memory. Any subsequent
+        calls to `.body`, `.form`, or `.json` will raise an error.
+        """
         if "body" in self.__dict__ and self.__dict__["body"].done():
             yield await self.body
             yield b""
@@ -126,13 +190,19 @@ class Request(HTTPConnection):
 
     @cached_property
     async def body(self) -> bytes:
-        chunks = []
-        async for chunk in self.stream():
-            chunks.append(chunk)
-        return b"".join(chunks)
+        """
+        Read all the contents of the request body into the memory and return it.
+        """
+        return b"".join([chunk async for chunk in self.stream()])
 
     @cached_property
     async def json(self) -> Any:
+        """
+        Call `await self.body` and use `json.loads` parse it.
+
+        If `content_type` is not equal to `application/json`,
+        an HTTPExcption exception will be thrown.
+        """
         if self.content_type == "application/json":
             data = await self.body
             return json.loads(
@@ -143,107 +213,108 @@ class Request(HTTPConnection):
 
     @cached_property
     async def form(self) -> FormData:
+        """
+        Parse the data in the form format and return it as a multi-value mapping.
+
+        If `content_type` is equal to `multipart/form-data`, it will directly
+        perform streaming analysis, and subsequent calls to `self.body`
+        or `self.json` will raise errors.
+
+        If `content_type` is not equal to `multipart/form-data` or
+        `application/x-www-form-urlencoded`, an HTTPExcption exception will be thrown.
+        """
         if self.content_type == "multipart/form-data":
             return await AsyncMultiPartParser(self.content_type, self.stream()).parse()
         if self.content_type == "application/x-www-form-urlencoded":
             data = (await self.body).decode(
                 encoding=self.content_type.options.get("charset", "latin-1")
             )
-            # this is type check error in mypy
-            return FormData(parse_qsl(data, keep_blank_values=True))  # type: ignore
+            return FormData(parse_qsl(data, keep_blank_values=True))
 
         raise HTTPException(
             415, {"Accpet": "multipart/form-data, application/x-www-form-urlencoded"}
         )
 
     async def close(self) -> None:
+        """
+        Close all temporary files in the `self.form`.
+
+        This can always be called, regardless of whether you use form or not.
+        """
         if "form" in self.__dict__ and self.__dict__["form"].done():
             await (await self.form).aclose()
 
     async def is_disconnected(self) -> bool:
+        """
+        The method used to determine whether the connection is interrupted.
+        """
         if not self._is_disconnected:
             try:
                 message = await asyncio.wait_for(self._receive(), timeout=0.0000001)
+                self._is_disconnected = message.get("type") == "http.disconnect"
             except asyncio.TimeoutError:
-                message = {}
-
-            self._is_disconnected = message.get("type") == "http.disconnect"
-
+                pass
         return self._is_disconnected
 
 
 class Response(BaseResponse):
+    """
+    The parent class of all responses, whose objects can be used directly as ASGI
+    application.
+    """
+
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
-        await send(
-            {
-                "type": "http.response.start",
-                "status": self.status_code,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in self.raw_headers
-                ],
-            }
-        )
-        await send({"type": "http.response.body", "body": b""})
+        self.headers["content-length"] = "0"
+        await send_http_start(send, self.status_code, self.list_headers(as_bytes=True))
+        return await send_http_body(send)
 
 
-ResponseContent = TypeVar("ResponseContent")
+_ContentType = TypeVar("_ContentType")
 
 
-class SmallResponse(Generic[ResponseContent], Response):
+class SmallResponse(Response, abc.ABC, Generic[_ContentType]):
+    """
+    Abstract base class for small response objects.
+    """
+
     media_type: str = ""
     charset = "utf-8"
 
     def __init__(
-        self, content: ResponseContent, status_code: int = 200, headers: dict = None
+        self,
+        content: _ContentType,
+        status_code: int = 200,
+        headers: Mapping[str, str] = None,
+        media_type: str = None,
+        charset: str = None,
     ) -> None:
         super().__init__(status_code, headers)
-        self.body = self.render(content)
-        self.generate_more_headers()
+        self.content = content
+        self.media_type = media_type or self.media_type
+        self.charset = charset or self.charset
 
-    def render(self, content: ResponseContent) -> bytes:
+    @abc.abstractmethod
+    async def render(self, content: _ContentType) -> bytes:
         raise NotImplementedError
 
-    def generate_more_headers(self) -> None:
-        body = getattr(self, "body", b"")
-        if body and not any(k == "content-length" for k, _ in self.raw_headers):
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        body = await self.render(self.content)
+        if body and "content-length" not in self.headers:
             content_length = str(len(body))
-            self.raw_headers.append(("content-length", content_length))
-
+            self.headers["content-length"] = content_length
         content_type = self.media_type
-        if content_type and not any(k == "content-type" for k, _ in self.raw_headers):
+        if content_type and "content-type" not in self.headers:
             if content_type.startswith("text/"):
                 content_type += "; charset=" + self.charset
-            self.raw_headers.append(("content-type", content_type))
-
-    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
-        await send(
-            {
-                "type": "http.response.start",
-                "status": self.status_code,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in self.raw_headers
-                ],
-            }
-        )
-        await send({"type": "http.response.body", "body": self.body})
+            self.headers["content-type"] = content_type
+        await send_http_start(send, self.status_code, self.list_headers(as_bytes=True))
+        await send_http_body(send, body)
 
 
 class PlainTextResponse(SmallResponse[Union[bytes, str]]):
     media_type = "text/plain"
 
-    def __init__(
-        self,
-        content: Union[bytes, str],
-        status_code: int = 200,
-        headers: dict = None,
-        media_type: str = "",
-    ) -> None:
-        self.media_type = media_type or self.media_type
-        super().__init__(content, status_code, headers)
-
-    def render(self, content: Union[bytes, str]) -> bytes:
+    async def render(self, content: Union[bytes, str]) -> bytes:
         return content if isinstance(content, bytes) else content.encode(self.charset)
 
 
@@ -251,233 +322,176 @@ class HTMLResponse(PlainTextResponse):
     media_type = "text/html"
 
 
-class JSONResponse(SmallResponse[JSONable]):
+class JSONResponse(SmallResponse[Any]):
+    """
+    `**kwargs` is used to accept all the parameters that `json.loads` can accept.
+    """
+
     media_type = "application/json"
 
     def __init__(
         self,
-        content: JSONable,
+        content: Any,
         status_code: int = 200,
-        headers: dict = None,
-        *,
-        ensure_ascii: bool = False,
-        allow_nan: bool = False,
-        indent: Union[int, str] = None,
-        separators: Optional[Tuple[str, str]] = (",", ":"),
-        default: Callable[[Any], Any] = None,
+        headers: Mapping[str, str] = None,
         **kwargs: Any,
     ) -> None:
-        self.json_kwargs = {
-            "ensure_ascii": ensure_ascii,
-            "allow_nan": allow_nan,
-            "indent": indent,
-            "separators": separators,
-            "default": default,
-            **kwargs,
+        self.json_kwargs: Dict[str, Any] = {
+            "ensure_ascii": False,
+            "allow_nan": False,
+            "indent": None,
+            "separators": (",", ":"),
+            "default": None,
         }
+        self.json_kwargs.update(**kwargs)
         super().__init__(content, status_code=status_code, headers=headers)
 
-    def render(self, content: JSONable) -> bytes:
-        # This is mypy error
-        return json.dumps(content, **self.json_kwargs).encode("utf-8")  # type: ignore
+    async def render(self, content: Any) -> bytes:
+        return json.dumps(content, **self.json_kwargs).encode(self.charset)
 
 
 class RedirectResponse(Response):
     def __init__(
-        self, url: Union[str, URL], status_code: int = 307, headers: dict = None
+        self,
+        url: Union[str, URL],
+        status_code: int = 307,
+        headers: Mapping[str, str] = None,
     ) -> None:
         super().__init__(status_code=status_code, headers=headers)
-        self.raw_headers.append(
-            ("location", quote_plus(str(url), safe=":/%#?&=@[]!$&'()*+,;"))
-        )
+        self.headers["location"] = quote_plus(str(url), safe=":/%#?&=@[]!$&'()*+,;")
 
 
 class StreamResponse(Response):
     def __init__(
         self,
-        generator: AsyncGenerator[bytes, None],
+        iterable: AsyncIterable[bytes],
         status_code: int = 200,
         headers: Mapping[str, str] = None,
         content_type: str = "application/octet-stream",
     ) -> None:
-        self.generator = generator
+        self.iterable = iterable
         super().__init__(status_code, headers)
-        self.raw_headers.append(("content-type", content_type))
-        self.raw_headers.append(("transfer-encoding", "chunked"))
+        self.headers["content-type"] = content_type
+        self.headers["transfer-encoding"] = "chunked"
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
-        await send(
-            {
-                "type": "http.response.start",
-                "status": self.status_code,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in self.raw_headers
-                ],
-            }
-        )
-        async for chunk in self.generator:
-            await send({"type": "http.response.body", "body": chunk, "more_body": True})
-
-        await send({"type": "http.response.body", "body": b"", "more_body": False})
+        await send_http_start(send, self.status_code, self.list_headers(as_bytes=True))
+        async for chunk in self.iterable:
+            await send_http_body(send, chunk, more_body=True)
+        return await send_http_body(send)
 
 
 class FileResponse(BaseFileResponse, Response):
+    """
+    File response.
+
+    It will automatically determine whether to send only headers
+    and the range of files that need to be sent.
+    """
+
     async def handle_all(
         self,
         send_header_only: bool,
         file_size: int,
-        headers: MutableSequence[Tuple[str, str]],
-        loop: asyncio.AbstractEventLoop,
         send: Send,
     ) -> None:
-        headers.append(("content-type", str(self.media_type)))
-        headers.append(("content-length", str(file_size)))
-        await send(
-            {
-                "type": "http.response.start",
-                "status": 200,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in chain(self.raw_headers, headers)
-                ],
-            }
-        )
+        self.headers["content-type"] = str(self.content_type)
+        self.headers["content-length"] = str(file_size)
+        await send_http_start(send, 200, self.list_headers(as_bytes=True))
         if send_header_only:
-            return await send({"type": "http.response.body", "body": b""})
+            return await send_http_body(send)
 
-        file = await loop.run_in_executor(None, open, self.filepath, "rb")
+        file = typing_cast(FileIO, await run_in_threadpool(open, self.filepath, "rb"))
         try:
-            for _ in range(0, file_size, 4096):
-                await send(
-                    {
-                        "type": "http.response.body",
-                        "body": await loop.run_in_executor(None, file.read, 4096),
-                        "more_body": True,
-                    }
+            for _ in range(0, file_size, self.chunk_size):
+                await send_http_body(
+                    send,
+                    await run_in_threadpool(file.read, self.chunk_size),
+                    more_body=True,
                 )
-            await send({"type": "http.response.body", "body": b""})
+            return await send_http_body(send)
         finally:
-            await loop.run_in_executor(None, file.close)
+            await run_in_threadpool(file.close)
 
     async def handle_single_range(
         self,
         send_header_only: bool,
         file_size: int,
-        headers: MutableSequence[Tuple[str, str]],
-        loop: asyncio.AbstractEventLoop,
         send: Send,
         start: int,
         end: int,
     ) -> None:
-        headers.append(("content-range", f"bytes {start}-{end-1}/{file_size}"))
-        headers.append(("content-length", str(end - start)))
-        await send(
-            {
-                "type": "http.response.start",
-                "status": 206,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in chain(self.raw_headers, headers)
-                ],
-            }
-        )
+        self.headers["content-range"] = f"bytes {start}-{end-1}/{file_size}"
+        self.headers["content-type"] = str(self.content_type)
+        self.headers["content-length"] = str(end - start)
+        await send_http_start(send, 206, self.list_headers(as_bytes=True))
         if send_header_only:
-            return await send({"type": "http.response.body", "body": b""})
+            return await send_http_body(send)
 
-        file = await loop.run_in_executor(None, open, self.filepath, "rb")
+        file = typing_cast(FileIO, await run_in_threadpool(open, self.filepath, "rb"))
         try:
-            await loop.run_in_executor(None, file.seek, start)
-            for here in range(start, end, 4096):
-                await send(
-                    {
-                        "type": "http.response.body",
-                        "body": await loop.run_in_executor(
-                            None, file.read, min(4096, end - here)
-                        ),
-                        "more_body": True,
-                    }
+            await run_in_threadpool(file.seek, start)
+            for here in range(start, end, self.chunk_size):
+                await send_http_body(
+                    send,
+                    await run_in_threadpool(
+                        file.read, min(self.chunk_size, end - here)
+                    ),
+                    more_body=True,
                 )
-            await send({"type": "http.response.body", "body": b""})
+            return await send_http_body(send)
         finally:
-            await loop.run_in_executor(None, file.close)
+            await run_in_threadpool(file.close)
 
     async def handle_several_ranges(
         self,
         send_header_only: bool,
         file_size: int,
-        headers: MutableSequence[Tuple[str, str]],
-        loop: asyncio.AbstractEventLoop,
         send: Send,
         ranges: Sequence[Tuple[int, int]],
     ) -> None:
-        headers.append(("content-type", "multipart/byteranges; boundary=3d6b6a416f9b5"))
+        self.headers["content-type"] = "multipart/byteranges; boundary=3d6b6a416f9b5"
         content_length = (
             18
-            + len(ranges) * (57 + len(self.media_type) + len(str(file_size)))
+            + len(ranges) * (57 + len(self.content_type) + len(str(file_size)))
             + sum(len(str(start)) + len(str(end - 1)) for start, end in ranges)
         ) + sum(end - start for start, end in ranges)
-        headers.append(("content-length", str(content_length)))
-        await send(
-            {
-                "type": "http.response.start",
-                "status": 206,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in chain(self.raw_headers, headers)
-                ],
-            }
-        )
+        self.headers["content-length"] = str(content_length)
+        await send_http_start(send, 206, self.list_headers(as_bytes=True))
         if send_header_only:
-            return await send({"type": "http.response.body", "body": b""})
+            return await send_http_body(send)
 
-        file = await loop.run_in_executor(None, open, self.filepath, "rb")
+        file = typing_cast(FileIO, await run_in_threadpool(open, self.filepath, "rb"))
         try:
             for start, end in ranges:
-                range_header = (
-                    "--3d6b6a416f9b5\n"
-                    f"Content-Type: {self.media_type}\n"
-                    f"Content-Range: bytes {start}-{end-1}/{file_size}\n\n"
-                ).encode("latin-1")
-                await send(
-                    {
-                        "type": "http.response.body",
-                        "body": range_header,
-                        "more_body": True,
-                    }
+                await send_http_body(
+                    send,
+                    (
+                        "--3d6b6a416f9b5\n"
+                        f"Content-Type: {self.content_type}\n"
+                        f"Content-Range: bytes {start}-{end-1}/{file_size}\n\n"
+                    ).encode("latin-1"),
+                    more_body=True,
                 )
-                await loop.run_in_executor(None, file.seek, start)
-                for here in range(start, end, 4096):
-                    await send(
-                        {
-                            "type": "http.response.body",
-                            "body": await loop.run_in_executor(
-                                None, file.read, min(4096, end - here)
-                            ),
-                            "more_body": True,
-                        }
+                await run_in_threadpool(file.seek, start)
+                for here in range(start, end, self.chunk_size):
+                    await send_http_body(
+                        send,
+                        await run_in_threadpool(
+                            file.read, min(self.chunk_size, end - here)
+                        ),
+                        more_body=True,
                     )
-                await send(
-                    {"type": "http.response.body", "body": b"\n", "more_body": True}
-                )
-            await send(
-                {
-                    "type": "http.response.body",
-                    "body": b"--3d6b6a416f9b5--\n",
-                    "more_body": True,
-                }
-            )
-            await send({"type": "http.response.body", "body": b""})
+                await send_http_body(send, b"\n", more_body=True)
+            return await send_http_body(send, b"--3d6b6a416f9b5--\n", more_body=False)
         finally:
-            await loop.run_in_executor(None, file.close)
+            await run_in_threadpool(file.close)
 
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         send_header_only = scope["method"] == "HEAD"
 
-        loop = asyncio.get_running_loop()
         stat_result = self.stat_result
         file_size = stat_result.st_size
-        headers = self.generate_required_header(stat_result)
 
         http_range, http_if_range = "", ""
         for key, value in scope["headers"]:
@@ -489,26 +503,37 @@ class FileResponse(BaseFileResponse, Response):
         if http_range == "" or (
             http_if_range != "" and not self.judge_if_range(http_if_range, stat_result)
         ):
-            return await self.handle_all(
-                send_header_only, file_size, headers, loop, send
-            )
+            return await self.handle_all(send_header_only, file_size, send)
 
-        ranges = self.parse_range(http_range, file_size)
+        try:
+            ranges = self.parse_range(http_range, file_size)
+        except HTTPException as exception:
+            await send_http_start(
+                send,
+                exception.status_code,
+                [
+                    (k.encode("latin-1"), v.encode("latin-1"))
+                    for k, v in (exception.headers or {}).items()
+                ],
+            )
+            return await send_http_body(send, exception.content or b"")
 
         if len(ranges) == 1:
             start, end = ranges[0]
             return await self.handle_single_range(
-                send_header_only, file_size, headers, loop, send, start, end
+                send_header_only, file_size, send, start, end
             )
         else:
             return await self.handle_several_ranges(
-                send_header_only, file_size, headers, loop, send, ranges
+                send_header_only, file_size, send, ranges
             )
 
 
 class SendEventResponse(Response):
     """
-    Server-sent events
+    Server-sent events response.
+
+    :param ping_interval: This determines the time interval (in seconds) between sending ping messages.
     """
 
     required_headers = {
@@ -519,7 +544,7 @@ class SendEventResponse(Response):
 
     def __init__(
         self,
-        generator: AsyncGenerator[ServerSentEvent, None],
+        iterable: AsyncIterable[ServerSentEvent],
         status_code: int = 200,
         headers: Mapping[str, str] = None,
         *,
@@ -532,7 +557,7 @@ class SendEventResponse(Response):
             headers = dict(self.required_headers)
         headers["Content-Type"] += f"; charset={charset}"
         super().__init__(status_code, headers)
-        self.generator = generator
+        self.iterable = iterable
         self.ping_interval = ping_interval
         self.charset = charset
 
@@ -541,47 +566,31 @@ class SendEventResponse(Response):
             {
                 "type": "http.response.start",
                 "status": self.status_code,
-                "headers": [
-                    (k.encode("latin-1"), v.encode("latin-1"))
-                    for k, v in self.raw_headers
-                ],
+                "headers": self.list_headers(as_bytes=True),
             }
         )
 
         done, pending = await asyncio.wait(
-            (self.keep_alive(send), self.send_event(send)),
+            (
+                asyncio.ensure_future(self.keep_alive(send)),
+                asyncio.ensure_future(self.send_event(send)),
+            ),
             return_when=asyncio.FIRST_COMPLETED,
         )
         [task.cancel() for task in pending]
         [task.result() for task in done]
-        await send({"type": "http.response.body", "body": b""})
+        return await send_http_body(send)
 
     async def send_event(self, send: Send) -> None:
-        async for chunk in self.generator:
-            if "data" in chunk:
-                data = (
-                    f"data: {_}".encode(self.charset)
-                    for _ in chunk.pop("data").splitlines()
-                )
-            event = b"\n".join(
-                chain(
-                    (f"{k}: {v}".encode(self.charset) for k, v in chunk.items()),
-                    data,
-                    (b"", b""),  # for generate b"\n\n"
-                )
-            )
-            await send({"type": "http.response.body", "body": event, "more_body": True})
+        async for chunk in self.iterable:
+            body = build_bytes_from_sse(chunk, self.charset)
+            await send_http_body(send, body, more_body=True)
 
     async def keep_alive(self, send: Send) -> None:
         while True:
             await asyncio.sleep(self.ping_interval)
-            await send(
-                {
-                    "type": "http.response.body",
-                    "body": ": ping\n\n".encode(self.charset),
-                    "more_body": True,
-                }
-            )
+            ping = b": ping\n\n"
+            await send_http_body(send, ping, more_body=True)
 
 
 class WebSocketDisconnect(Exception):
@@ -646,6 +655,9 @@ class WebSocket(HTTPConnection):
             raise RuntimeError('Cannot call "send" once a close message has been sent.')
 
     async def accept(self, subprotocol: str = None) -> None:
+        """
+        Accept websocket connection.
+        """
         if self.client_state == WebSocketState.CONNECTING:
             # If we haven't yet seen the 'connect' message, then wait for it first.
             await self.receive()
@@ -656,18 +668,27 @@ class WebSocket(HTTPConnection):
             raise WebSocketDisconnect(message["code"])
 
     async def receive_text(self) -> str:
+        """
+        Receive a WebSocket text frame and return.
+        """
         assert self.application_state == WebSocketState.CONNECTED
         message = await self.receive()
         self._raise_on_disconnect(message)
         return message["text"]
 
     async def receive_bytes(self) -> bytes:
+        """
+        Receive a WebSocket binary frame and return.
+        """
         assert self.application_state == WebSocketState.CONNECTED
         message = await self.receive()
         self._raise_on_disconnect(message)
         return message["bytes"]
 
     async def iter_text(self) -> AsyncIterator[str]:
+        """
+        Keep receiving text frames until the WebSocket connection is disconnected.
+        """
         try:
             while True:
                 yield await self.receive_text()
@@ -675,6 +696,9 @@ class WebSocket(HTTPConnection):
             pass
 
     async def iter_bytes(self) -> AsyncIterator[bytes]:
+        """
+        Keep receiving binary frames until the WebSocket connection is disconnected.
+        """
         try:
             while True:
                 yield await self.receive_bytes()
@@ -682,17 +706,36 @@ class WebSocket(HTTPConnection):
             pass
 
     async def send_text(self, data: str) -> None:
+        """
+        Send a WebSocket text frame.
+        """
         await self.send({"type": "websocket.send", "text": data})
 
     async def send_bytes(self, data: bytes) -> None:
+        """
+        Send a WebSocket binary frame.
+        """
         await self.send({"type": "websocket.send", "bytes": data})
 
     async def close(self, code: int = 1000) -> None:
+        """
+        Close WebSocket connection. It can be called multiple times.
+        """
         if self.application_state != WebSocketState.DISCONNECTED:
             await self.send({"type": "websocket.close", "code": code})
 
 
 def request_response(view: Callable[[Request], Awaitable[Response]]) -> ASGIApp:
+    """
+    This can turn a callable object into a ASGI application.
+
+    ```python
+    @request_response
+    async def f(request: Request) -> Response:
+        ...
+    ```
+    """
+
     @functools.wraps(view)
     async def asgi(scope: Scope, receive: Receive, send: Send) -> None:
         request = Request(scope, receive, send)
@@ -703,6 +746,22 @@ def request_response(view: Callable[[Request], Awaitable[Response]]) -> ASGIApp:
 
 
 class Router(BaseRouter[ASGIApp]):
+    """
+    A router to assign different paths to different ASGI applications.
+
+    :param routes: A triple composed of path, endpoint, and name. The name is optional. \
+        If the name is not given, the corresponding URL cannot be constructed through \
+        build_url.
+
+    ```python
+    applications = Router(
+        ("/static/{filepath:path}", static_files),
+        ("/api/{any:path}", api_app),
+        ("/about/{name}", about_page, "about"),
+        ("/", homepage, "homepage"),
+    )
+    """
+
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         path = scope["path"]
         for route in self._route_array:
@@ -716,7 +775,46 @@ class Router(BaseRouter[ASGIApp]):
         return await PlainTextResponse(b"", 404)(scope, receive, send)
 
 
+class Subpaths(BaseSubpaths[ASGIApp]):
+    """
+    A router allocates different prefix requests to different ASGI applications.
+
+    NOTE: This will change the values of `scope["root_path"]` and `scope["path"]`.
+
+    ```python
+    applications = Subpaths(
+        ("/static", static_files),
+        ("/api", api_app),
+        ("", default_app),
+    )
+    ```
+    """
+
+    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
+        path = scope["path"]
+        for prefix, endpoint in self._route_array:
+            if not path.startswith(prefix):
+                continue
+            scope["root_path"] = scope.get("root_path", "") + prefix
+            scope["path"] = path[len(prefix) :]
+            return await endpoint(scope, receive, send)
+
+        return await PlainTextResponse(b"", 404)(scope, receive, send)
+
+
 class Hosts(BaseHosts[ASGIApp]):
+    r"""
+    A router that distributes requests to different ASGI applications based on Host.
+
+    ```python
+    applications = Hosts(
+        (r"static\.example\.com", static_files),
+        (r"api\.example\.com", api_app),
+        (r"(www\.)?example\.com", default_app),
+    )
+    ```
+    """
+
     async def __call__(self, scope: Scope, receive: Receive, send: Send) -> None:
         host = ""
         for k, v in scope["headers"]:
